import time
import functools
from django.views.decorators.csrf import csrf_exempt
from django.shortcuts import render
from django.http import JsonResponse
from .annotations import Integer
from .annotations import String
from .annotations import Bytes
from .annotations import Boolean
from .utils import DjangoJsonEncoder
from .utils import request_get_dict
from .utils import get_payload
from .utils import get_inject_params


@csrf_exempt
def apiview(view):
    @functools.wraps(view)
    def wrapper(request, *args, **kwargs):
        package = {}
        try:
            data = request_get_dict(request)
            data["_request"] = request
            form = get_payload(request)
            data["_form"] = form
            for key, value in form:
                if not key in data:
                    data[key] = value
            if not "request" in data:
                data["request"] = request
            if not "form" in data:
                data["form"] = form
            params = get_inject_params(view, data)
            result = view(**params)
            package["success"] = True
            package["result"] = result
        except Exception as error:
            package["success"] = False
            package["error"] = error
        return JsonResponse(package, encoder=DjangoJsonEncoder, json_dumps_params={"ensure_ascii": False})
    return wrapper


# ===========================================
# test views
# ===========================================
@apiview
def ping():
    return "pong"

@apiview
def timestamp():
    return int(time.time())

@apiview
def echo(msg):
    return msg

@apiview
def getBooleanResult(value : Boolean):
    return value

@apiview
def getIntegerResult(value: Integer):
    return value

@apiview
def getBytesResult(value: Bytes):
    return value



import base64
import binascii
from fastutils import strutils

STRING_ENCODINGS = ["utf-8", "gb18030"]


def Boolean(obj):
    if isinstance(obj, bool):
        return obj
    if isinstance(obj, str):
        if obj.lower() in ["true", "yes", "y", "1"]:
            return True
        else:
            return False
    if isinstance(obj, int):
        if obj != 0:
            return True
        else:
            return False
    if obj:
        return True
    else:
        return False


def Integer(obj):
    if isinstance(obj, int):
        return obj
    else:
        return int(obj)

def String(obj):
    if isinstance(obj, bytes):
        try:
            return obj.decode()
        except UnicodeDecodeError:
            for encoding in STRING_ENCODINGS:
                try:
                    return obj.decode(encoding)
                except UnicodeDecodeError:
                    pass
    return str(obj)


def Bytes(obj):
    if isinstance(obj, bytes):
        return obj
    if isinstance(obj, str):
        if strutils.is_unhexlifiable(obj):
            return binascii.unhexlify(obj)
        elif strutils.is_base64_decodable(obj):
            return base64.decodebytes(obj.encode())
        elif strutils.is_urlsafeb64_decodable(obj):
            return base64.urlsafe_b64decode(obj.encode())
        else:
            return obj.encode("utf-8")
    obj = str(obj)
    return obj.encode("utf-8")



import os
import urllib
import binascii
import json
import requests
from django.test import LiveServerTestCase
from apiview.annotations import Bytes

class TestViews(LiveServerTestCase):

    def setUp(self):
        self.url = self.live_server_url

    def get_url(self, url):
        return urllib.parse.urljoin(self.live_server_url, url)
    
    def test01(self):
        url = self.get_url("/test/ping")
        response = requests.get(url)
        data = json.loads(response.content)
        assert data["result"] == "pong"

    def test02(self):
        url = self.get_url("/test/timestamp")
        response = requests.get(url)
        data = json.loads(response.content)
        assert isinstance(data["result"], int)

    def test03(self):
        url = self.get_url("/test/echo")
        params = {
            "msg": "hello world"
        }
        response = requests.get(url, params=params)
        data = json.loads(response.content)
        assert data["result"] == params["msg"]

    def test04(self):
        url = self.get_url("/test/getBooleanResult")
        params = {
            "value": "yes"
        }
        response = requests.get(url, params=params)
        data = json.loads(response.content)
        assert data["result"] is True

    def test05(self):
        url = self.get_url("/test/getBooleanResult")
        params = {
            "value": False
        }
        response = requests.get(url, params=params)
        data = json.loads(response.content)
        assert data["result"] is False

    def test06(self):
        url = self.get_url("/test/getIntegerResult")
        params = {
            "value": 123
        }
        response = requests.get(url, params=params)
        data = json.loads(response.content)
        assert data["result"] == 123


    def test07(self):
        url = self.get_url("/test/getIntegerResult")
        params = {
            "value": "123"
        }
        response = requests.get(url, params=params)
        data = json.loads(response.content)
        assert data["result"] == 123

    def test08(self):
        url = self.get_url("/test/getBytesResult")
        bts = os.urandom(32)
        params = {
            "value": binascii.hexlify(bts).decode(),
        }
        response = requests.get(url, params=params)
        data = json.loads(response.content)
        assert Bytes(data["result"]) == bts




import inspect
import json
import binascii
import bizerror
from django.db import models
from django.core.serializers.json import DjangoJSONEncoder as DjangoJSONEncoderBase
from django.forms.models import model_to_dict
from django.core import serializers

def model_to_json(o):
    text = serializers.serialize("json", [o])
    results = json.loads(text)
    obj = results[0]["fields"]
    obj["pk"] = results[0]["pk"]
    return obj

def queryset_to_json(q):
    text = serializers.serialize("json", q)
    results = json.loads(text)
    data = []
    for result in results:
        obj = result["fields"]
        obj["pk"] = result["pk"]
        data.append(obj)
    return data

class DjangoJsonEncoder(DjangoJSONEncoderBase):

    def default(self, o):
        if isinstance(o, bytes):
            return binascii.hexlify(o).decode()
        if isinstance(o, models.Model):
            return model_to_json(o)
        if isinstance(o, models.QuerySet):
            return queryset_to_json(o)
        if isinstance(o, bizerror.BizError):
            return o.json
        if isinstance(o, Exception):
            return bizerror.UndefinedError(str(o)).json
        return super().default(o)

def request_get_dict(request):
    data = {}
    for name, _ in request.GET.items():
        value = request.GET.getlist(name)
        if isinstance(value, (list, tuple, set)) and len(value) == 1:
            data[name] = value[0]
        else:
            data[name] = value
    return data

def get_payload(request):
    if request.body:
        try:
            return json.loads(request.body)
        except:
            pass
    return {}

def get_inject_params(func, data):
    params = {}
    parameters = inspect.signature(func).parameters
    for name, parameter in parameters.items():
        if parameter.default is parameter.empty: # 没有默认值，为必选参数
            value = data[name]
        else:
            value = data.get(name, parameter.default)
        if not parameter.annotation is parameter.empty:
            value = parameter.annotation(value)
        params[name] = value
    return params



import os
import binascii
import base64
from django.test import TestCase
from django.contrib.auth.models import User
from apiview.utils import model_to_json
from apiview.utils import queryset_to_json
from apiview.annotations import Boolean
from apiview.annotations import Integer
from apiview.annotations import String
from apiview.annotations import Bytes

class TestApiview(TestCase):

    def test01(self):
        u = User()
        u.username = 'test01'
        u.email = 'test01@example.com'
        u.save()
        data = model_to_json(u)
        assert data
        assert data["username"] == "test01"
    
    def test02(self):
        u = User()
        u.username = "test02"
        u.email = "test02@example.com"
        u.save()
        us = User.objects.all()
        data = queryset_to_json(us)
        assert data
        assert isinstance(data, list)

    def test03(self):
        assert Boolean("True") is True
        assert Boolean("False") is False
        assert Boolean("1") is True
        assert Boolean("0") is False
        assert Boolean("y") is True
        assert Boolean("n") is False
        assert Boolean("yes") is True
        assert Boolean("no") is False
        assert Boolean(1) is True
        assert Boolean(0) is False
        assert Boolean(True) is True
        assert Boolean(False) is False
        assert Boolean(1.1) is True
        assert Boolean(0.0) is False

    def test04(self):
        assert Integer(1) == 1
        assert Integer(0) == 0
        assert Integer("1") == 1
        assert Integer("0") == 0

    def test05(self):
        assert String("a") == "a"
        assert String("测试") == "测试"
        assert String(1) == "1"
        assert String(True) == "True"
        assert String(False) == "False"
        assert String("测试".encode("utf-8")) == "测试"
        assert String("测试".encode("gbk")) == "测试"

    def test06(self):
        assert Bytes("a") == b"a"
        assert Bytes(b"a") == b"a"
        assert Bytes("测试") == "测试".encode("utf-8")
        assert Bytes("YQ==") == b"a"
        assert Bytes("YWI=") == b"ab"
        assert Bytes("6162") == b"ab"

    def test07(self):
        s = os.urandom(16)
        t1 = binascii.hexlify(s).decode()
        t2 = base64.encodebytes(s).decode()
        t3 = base64.urlsafe_b64encode(s).decode()
        assert Bytes(t1) == s
        assert Bytes(t2) == s
        assert Bytes(t3) == s



from django.urls import path
from . import views


urlpatterns = [
    path('ping', views.ping),
    path('timestamp', views.timestamp),
    path('echo', views.echo),
    path('getBooleanResult', views.getBooleanResult),
    path('getIntegerResult', views.getIntegerResult),
    path('getBytesResult', views.getBytesResult),
]
